/*
 *   DVIBASE section
 */

#include <stdio.h>
#include <stdio.h>
#include "dvicom.h"
#include "dvistr.h"
#include "dstypes.h"
#include "xmlsvg.h"
#include "string.h"

/*
 * used in dvipage.c
 */
int post_s;

/*
 * units (mm = milimeter, pt = point)
 * unit_pt used in svg_svg, DVI_FNT_DEF_1
 *         used in svgout.c
 */
float unit_pt;

LOCAL dvi_pre_struct pre;
LOCAL void dvi_pre(FILE *dvi_inp, FILE *log_out);

LOCAL dvi_post_post_struct post_post;
LOCAL int dvi_post_post(FILE *dvi_inp, FILE *log_out);

LOCAL dvi_post_struct post;
LOCAL void dvi_post(FILE *dvi_inp, FILE *log_out);

LOCAL void fontinit();
LOCAL void fontfree(FILE* log_out);
LOCAL int dvi_fnt_def(FILE *dvi_inp, FILE *svg_out, FILE *log_out);
LOCAL void svgfontentout(FILE *svg_out, FILE *log_out);
LOCAL void svgfontstlout(char *svgfontsfile, FILE *svg_out, FILE *log_out);

LOCAL dvi_bop_struct bop;
LOCAL WORD bop_p_old;
LOCAL int dvi_bop(FILE *dvi_inp, FILE *log_out);

/*
 *  *fnt_def_k
 */
LOCAL dvi_fnt_def_k_struct *fnt_def_k[256];

/*
 * DVIBASE function
 */

int dvibase(char *svgfontsfile, FILE *dvi_inp, FILE *svg_out, FILE *log_out) {
	float unit_mm;

/*
 * DVI_PRE
 */
	if(getc(dvi_inp) != DVI_PRE) return 15;
	dvi_pre(dvi_inp, log_out);

	unit_mm = pre.num * 0.0001 / pre.den * (pre.mag / 1000.0);
	unit_pt = unit_mm * 72.27/25.4;

/*
 * DVI_POST_POST
 */
	if(dvi_post_post(dvi_inp, log_out) == 12) return 12;

/*
 * DVI_POST
 */
	fseek(dvi_inp, post_post.q, SEEK_SET);
	if(getc(dvi_inp) != DVI_POST) return 13;
	dvi_post(dvi_inp, log_out);
	post_s = post.s;

/*
 *  post.p used in DVI_BOP
 *   for seeking on last page in DVI file
 *  post.s stack handling (used in dvipage.c)
 *  post.l and post.u used in SVG svg
 *   for width and height of page in SVG document
 *  now not used: post.p, post.t
 *   post.p is for DVI version in dvi file
 *   post.t is for number of pages in dvi file
 */

/*
 * DVI_FNT_DEF_i
 */

	printf("Fonts:");
/* SVG_OUT */
	fprintf(svg_out, xml_dtdsys, SVGDTDS);
	fontinit();
	if(dvi_fnt_def(dvi_inp, svg_out, log_out) == 14) return 14;
	printf("\n");
/* SVG_OUT */
	svgfontentout(svg_out, log_out);
	fprintf(svg_out, xml_dtdsys2);
	fprintf(svg_out, svg_svg, 
		(post.u * unit_pt) + HORIGIN, (post.l * unit_pt) + VORIGIN,
		SVGVER,
		(post.u * unit_pt) + HORIGIN, (post.l * unit_pt) + VORIGIN,
		SVGNS);
	fprintf(svg_out, stl_css);
	fprintf(svg_out, xml_cdata);
	svgfontstlout(svgfontsfile, svg_out, log_out);
	fprintf(svg_out, xml_cdata2);
	fprintf(svg_out, stl_css2);
	fontfree(log_out);
/*
 * DVI_BOP
 */
	bop.p = post.p;
	if(dvi_bop(dvi_inp, log_out) ==  16) return 16;
/*
 * We go on first page
 */
	fseek(dvi_inp, bop_p_old, SEEK_SET);

	return 0;
}

LOCAL void dvi_pre(FILE *dvi_inp, FILE *log_out) {
	int pp;
	fprintf(log_out, "#### dvibase.c\n");
	fprintf(log_out, "PRE: ");
	pre.i = napln1(dvi_inp);
	pre.num = napln4(dvi_inp);
	pre.den = napln4(dvi_inp);
	pre.mag = napln4(dvi_inp);
	fprintf(log_out, "i:%d num:%d den:%d mag:%d ",
 	       pre.i, pre.num, pre.den, pre.mag);
	pre.k = napln1(dvi_inp);
	pre.x = (char *) malloc(sizeof(char)*(pre.k + 1));
	for(pp = 0; pp < pre.k; pp++)
		pre.x[pp] = (char) napln1(dvi_inp);
	pre.x[pre.k] = '\0';
	fprintf(log_out, "\n     x:'%s'",pre.x);
	printf("'%s'\n",pre.x);
	free(pre.x);
	fprintf(log_out, "\n");
}

LOCAL int dvi_post_post(FILE *dvi_inp, FILE *log_out) {
/*
 * fseek use long for file position, in DVI is WORD
 */
	WORD dvi_file_pos;
/*
 * DVI_PADDING
 *    ALL i[1] != DVI_PADDING
 *    && ('file length' / n) == 0 [in TBN is n = 4]
 *    && padding 0..n-1
 */
	for(dvi_file_pos=-1; dvi_file_pos>=-8; dvi_file_pos--) {
		fseek(dvi_inp, dvi_file_pos, SEEK_END);
		if(getc(dvi_inp) != DVI_PADDING) {
			fprintf(log_out, "Padding:%d",-dvi_file_pos-1);
			break;
		}
	}

/*
 * DVI_POST_POST
 */
	fseek(dvi_inp, dvi_file_pos -= 5, SEEK_END);
	if(getc(dvi_inp) != DVI_POST_POST) return 12;
	post_post.q = napln4(dvi_inp);
	post_post.i = napln1(dvi_inp);
	fprintf(log_out, "\nPOST_POST: q:%d i:%d",
		post_post.q, post_post.i);
	return 0;
}

LOCAL void dvi_post(FILE *dvi_inp, FILE *log_out) {
	fprintf(log_out, "\nPOST: ");
	post.p = napln4(dvi_inp);
	fprintf(log_out, "p:%d ",post.p);
	post.num = napln4(dvi_inp); 
	post.den = napln4(dvi_inp);
	post.mag = napln4(dvi_inp);
	fprintf(log_out, "num:%d den:%d mag:%d\n",
	        post.num, post.den, post.mag);
	post.l = napln4(dvi_inp);
	post.u = napln4(dvi_inp);
	post.s = napln2(dvi_inp);
	post.t = napln2(dvi_inp);
	fprintf(log_out, "      l:%d u:%d s:%d t:%d",
	        post.l, post.u, post.s, post.t);
}

LOCAL void fontinit() {
	int pp;

	for(pp=0; pp<256; pp++) fnt_def_k[pp] = NULL;
}

LOCAL void svgfontentout(FILE *svg_out, FILE *log_out) {
	int pp;

	for(pp=0; pp<256; pp++)
	if(fnt_def_k[pp] != NULL) {
		fprintf(svg_out, xml_ent,
		pp, fnt_def_k[pp]->n, fnt_def_k[pp]->s * unit_pt);
	}
}

LOCAL void svgfontstlout(char *svgfontsfile, FILE *svg_out, FILE *log_out) {
	int pp;

	for(pp=0; pp<256; pp++)
	if(fnt_def_k[pp] != NULL) {
		fprintf(svg_out, stl_fntface,
		fnt_def_k[pp]->n, fnt_def_k[pp]->s * unit_pt,
		svgfontsfile, fnt_def_k[pp]->n);
	}
}

LOCAL void fontfree(FILE *log_out) {
	int pp;

	fprintf(log_out, "\ndvibasefontfree:", pp);
	for(pp=0; pp<256; pp++)
	if(fnt_def_k[pp] != NULL) {
			fprintf(log_out, " %d", pp);
			free(fnt_def_k[pp]->n);
			free(fnt_def_k[pp]);
	}
	fprintf(log_out, "\n");
}

LOCAL int dvi_fnt_def(FILE *dvi_inp, FILE *svg_out, FILE *log_out) {
	int fnt_c;
	dvi_fnt_def_struct fnt_def;
	int pp;
	WORD fnt_def_al;

	while((fnt_c = getc(dvi_inp)) != DVI_POST_POST)
	if(fnt_c == DVI_FNT_DEF_1) {
		fprintf(log_out, "\nFNT_DEF_1: ");
		fnt_def.k = napln1(dvi_inp); 
		fnt_def.c = napln4(dvi_inp); 
		fnt_def.s = napln4(dvi_inp); 
		fnt_def.d = napln4(dvi_inp); 
		fnt_def.a = napln1(dvi_inp); 
		fnt_def.l = napln1(dvi_inp); 
		fnt_def_al = fnt_def.l + fnt_def.a;
		fprintf(log_out,
			"k:%2d, c:%10u, s:%d, d:%d, a:%d, l:%d ",
			fnt_def.k, fnt_def.c, fnt_def.s,
			fnt_def.d, fnt_def.a, fnt_def.l);
		fnt_def.n = (char *)
			malloc(sizeof(char) * (fnt_def_al+1));
		for(pp = 0; pp < fnt_def_al; pp++)
			fnt_def.n[pp] = (char) napln1(dvi_inp);
		fnt_def.n[fnt_def_al] = '\0';
		printf("*");
		fprintf(log_out, "n:%s", fnt_def.n);
		fnt_def_k[fnt_def.k] =
			(dvi_fnt_def_k_struct *)
			malloc(sizeof(dvi_fnt_def_k_struct));
		fnt_def_k[fnt_def.k]->c = fnt_def.c;
		fnt_def_k[fnt_def.k]->s = fnt_def.s;
		fnt_def_k[fnt_def.k]->d = fnt_def.d;
		fnt_def_k[fnt_def.k]->n = (char *) malloc(strlen(fnt_def.n));
		strcpy(fnt_def_k[fnt_def.k]->n, fnt_def.n);

	free(fnt_def.n);
	}
	else
		return 14; 
	return 0;
}

LOCAL int dvi_bop(FILE *dvi_inp, FILE *log_out) {
	int pp;
	while(bop.p != -1) {
		fseek(dvi_inp, bop.p, SEEK_SET);
		if(getc(dvi_inp) != DVI_BOP) return 16;
		fprintf(log_out, "\nBOP: ");
		for(pp=0; pp<10; pp++)
			bop.c[pp] = napln4(dvi_inp);
		bop_p_old = bop.p;
		bop.p = napln4(dvi_inp);
		for(pp=0; pp<10; pp++)
			fprintf(log_out, "c%d:%d ", pp, bop.c[pp]);
		fprintf(log_out, "p:%d", bop.p);
	}
	fprintf(log_out, "\n");	
	return 0;
}

/*
 *   DVIBASE section
 *   vim:ts=4:
 */
